/*******************************************************

 *******************************************************/

import java.awt.*;
import java.awt.image.*;

public class Animation
{
   final static int INTARRAY = 1;
   final static int IMAGE = 2;
   final static int BYTEMASK = 4;

   Image[] images = null;
   int[][] arrays = null;
   byte[][] barrays = null;
   int[] flags = null;

   int width = 360;
   int height = 180;
   int count = 0;

   String name = null;

   public Animation(String animName, int count)
   {
      this.count = count;
      this.name = animName;

      initAnim();
   }

   public Animation(String animName, int count, int width, int height)
   {
      this.count = count;
      this.name = animName;
      this.width = width;
      this.height = height;

      initAnim();
   }

   private void initAnim()
   {
      images = new Image[count];
      arrays = new int[count][];
      barrays = new byte[count][];
      flags = new int[count];
   }


/*******************************************************

 *******************************************************/

   public int[] getIntFrame(int frame)
   {
      if ((flags[frame] & INTARRAY) != 0) return arrays[frame];

      if ((flags[frame] & IMAGE) != 0) return grabImage(images[frame]);

      return null;
   }


   public byte[] getByteMask(int frame)
   {
      if ((flags[frame] & BYTEMASK) != 0) return barrays[frame];
      if ((flags[frame] & INTARRAY) != 0) return grabMask(getIntFrame(frame));

      return null;
   }

   public Image getImageFrame(int frame)
   {
      if ((flags[frame] & IMAGE) != 0) return images[frame];

      if ((flags[frame] & INTARRAY) != 0)
      {
         System.err.println("makeImage(int[]) not implemented!");
      }

      return null;
   }

   public void setFrame(int frame, Image newImage, int flags)
   {
      if (newImage != null)
      {
         this.flags[frame] = flags;

         if ((flags & INTARRAY) != 0) arrays[frame] = grabImage(newImage);
         if ((flags & IMAGE) != 0) images[frame] = newImage;
         if ((flags & BYTEMASK) != 0)
         {
            if ((flags & INTARRAY) != 0) barrays[frame] = grabMask(getIntFrame(frame));
            else barrays[frame] = grabMask(grabImage(newImage));
         }
      }
   }

/*******************************************************

 *******************************************************/

   public byte[] grabMask(int[] temp)
   {
      return grabMask(temp, width*height);
   }

   public static byte[] grabMask(int[] temp, int count)
   {
      byte[] myMask = new byte[count];

      for(int i=0;i<count;i++)
      {
         myMask[i] = (byte) ((temp[i] & 0xff) >> 1);
      }

      return myMask;
   }

   public static int[] grabImage(Image source)
   {
      int w,h;

      if (source == null) return null;

      do {
         w = source.getWidth( null );
         h = source.getHeight( null );
      } while(( w == -1 ) || ( h == -1 ));

      int[] temp = new int[w*h];

      PixelGrabber grabber = new PixelGrabber(source.getSource(),
         0, 0, w, h, temp, 0, w);

      try {
         grabber.grabPixels();
      } catch (Exception e) {
         System.out.println("grabbbaz");
      };

      return temp;
   }
}