/********************************************************

   VectorApplet         (c) Mikko Sairio 1997

 ********************************************************/

import java.io.*;
import java.net.*;
import java.awt.*;
import java.awt.image.*;

public class Vectors extends java.applet.Applet implements Runnable
{
   private vWorld myWorld = null;

   private Image layerImage = null;
   private int layerW = 0;
   private int layerH = 0;
   private int layerX = 0;
   private int layerY = 0;

   private int[] layerData = null;

   private vAnim myAnim = null;
   int width = 0, height = 0;

   Thread myThread;

   String dobFile = "default.dob";

   public final int TOP = 1;
   public final int BOTTOM = 2;
   public final int LEFT = 4;
   public final int RIGHT = 8;

   URL codeBase = null;

   public void init() {

      super.init();

      setLayout( null );

      codeBase = getCodeBase();

      myWorld = new vWorld( size().width, size().height );

      width = size().width;
      height = size().height;

      String paramStr = getParameter("DOBFile");

      if (paramStr != null)
      {
         dobFile = paramStr;
      }

      paramStr = getParameter("bgcolor");

      if (paramStr != null)
      {
         int i = fromHex(paramStr);
         Color bgColor = new Color(i | 0xff000000);
         setBackground(bgColor);
         myWorld.setBGColor(i);
      }

      paramStr = getParameter("layer");

      if (paramStr != null)
      {
         try {
            layerImage = getImage(new URL( codeBase, paramStr ));
         } catch(Exception e) {}

         paramStr = getParameter("layeralign");

         do {
            layerW = layerImage.getWidth( null );
            layerH = layerImage.getHeight( null );
         } while(layerW == -1 || layerH == -1);

         layerData = new int[layerW*layerH];

         PixelGrabber grabber = new PixelGrabber(
            layerImage, 0, 0, layerW, layerH, layerData, 0, layerW );

         try {
            grabber.grabPixels();
         } catch (InterruptedException e) {}

         for(int i=0;i<layerW*layerH;i++)
            layerData[i] &= 255;

         layerX = (width - layerW) / 2;
         layerY = (height - layerH) / 2;

         if (paramStr != null)
         {
            if (paramStr.indexOf("top")!=-1)
            {
               layerY = 0;
            }
            if (paramStr.indexOf("bottom")!=-1)
            {
               layerY = height - layerH;
            }
            if (paramStr.indexOf("left")!=-1)
            {
               layerX = 0;
            }
            if (paramStr.indexOf("right")!=-1)
            {
               layerX = width - layerW;
            }
         }

         myWorld.layerData = layerData;
         myWorld.layerW = layerW;
         myWorld.layerH = layerH;
         myWorld.layerX = layerX;
         myWorld.layerY = layerY;

         int[] temp = new int[layerW*layerH];

         for(int i=0;i<layerW*layerH;i++)
            temp[i] = myWorld.BGInt;

         myWorld.mixLayer(temp, 0);

         MemoryImageSource s = new MemoryImageSource(
            layerW, layerH, temp, 0, layerW);

         layerImage = createImage(s);

         System.out.println("layerimage = " + layerImage + " layerX = " +
            layerX + " layerY = " + layerY + " layerW = " + layerW + " layerH = " + layerH);
      }
   }

/********************************************************

 ********************************************************/

   public int fromHex( String hexstr )
   {
      String hexrange = "0123456789abcdef0123456789ABCDEF";

      int i=0;
      int value=0;
      int num=0;

      while(i < hexstr.length() )
      {
         num = hexrange.indexOf(hexstr.charAt(i++));

         if (num == -1)
         {
            i = hexstr.length();
            value = -1;
         }
         else
         {
            value <<= 4;
            value += (num & 15);
         }
      }

      return value;
   }

/********************************************************

 ********************************************************/

   private int getIntParameter( String paramName )
   {
      String paramString = getParameter( paramName );
//      System.out.println( paramName + " = " + paramString );
      int val = -1;

      if (paramString != null) {
          try {
              val = Integer.parseInt( paramString );
          } catch(NumberFormatException e) {
            val = -1;
          }
      }

      return val;
   }

   public void start()
   {
      myThread = new Thread( this, "vector thread" );
      myThread.start();
   }

   public void stop()
   {
      myThread = null;
   }

/********************************************************

 ********************************************************/

   int animH = 0;

   public void paint( Graphics g )
   {
      if (animH >= height)
      {
         if (myAnim != null)
         {
            g.clipRect(0,0,width,height);
            g.drawImage( myAnim.getNextAvail(), 0, 0, null );
            myAnim.releaseSlot();
         }
      }
      else
      {
         if (myAnim != null)
         {
            Graphics tg = g.create();
            tg.clipRect(0,0,width,animH);
            tg.drawImage( myAnim.getNextAvail(), 0, 0, null );
            myAnim.releaseSlot();

            animH += 3;
         }

         g.clipRect(0,animH,width,height-animH);
         g.setColor( myWorld.BGColor );
         g.fillRect( 0,0,width,height );
         if (layerImage != null) g.drawImage( layerImage, layerX, layerY, null );
         g.setColor(Color.black);
         g.drawLine(width-1,0,width-1,height);
         g.drawLine(0,0,width,0);
      }
      frameNotDone = false;
   }

   public void update( Graphics g )
   {
      if (frameNotDone)
      {
         if (animH >= height)
         {
            if (myAnim != null)
            {
               g.clipRect(0,0,width,height);
               g.drawImage( myAnim.getNextAvail(), 0, 0, null );
               myAnim.releaseSlot();
            }
         }
         else
         {
            if (myAnim != null)
            {
               g.clipRect(0,0,width,animH);
               g.drawImage( myAnim.getNextAvail(), 0, 0, null );
               myAnim.releaseSlot();
               animH += 3;
            }

            g.setColor(Color.black);
            g.drawLine(width-1,0,width-1,height);
            g.drawLine(0,0,width,0);

         }

         frameNotDone = false;
      }
      else paint( g );
   }

/********************************************************

 ********************************************************/

   boolean frameNotDone = false;
   int frameIndex = 0;

/********************************************************

 ********************************************************/

   public void run()
   {
      repaint();

      if (!myWorld.isWorldLoaded())
      {
         URL DOBSource = null;
         DataInputStream DOBStream = null;

         try {
            DOBSource = new URL(codeBase, dobFile);
            DOBStream = new DataInputStream(new BufferedInputStream( DOBSource.openStream() ));
         } catch (Exception e)
         {
            System.out.println("Maailmatiedoston avaaminen eponnistui.");
         }

         myWorld.loadDOB( DOBStream );

         try {
            DOBStream.close();
         } catch (Exception e) {}
      }

      myAnim = new vAnim( vAnim.BUFFERED, 3, myWorld, this );
//      myAnim = new vAnim( vAnim.LOOPING, 100, myWorld, this );

      repaint();

      long sysTime = System.currentTimeMillis();
      long delay = 0;

      while(myThread != null)
      {
         while(frameNotDone)
         {
            try {
               Thread.sleep(20);
            } catch(InterruptedException e) {}
         }

         do {
            frameIndex++;
         } while((delay = (sysTime+=100) - System.currentTimeMillis()) < 10);

         try { Thread.sleep(delay); } catch (InterruptedException e) {};

         frameNotDone = true;
         repaint();
      }

      myAnim.animThread = null;
      myAnim = null;
   }

   public String[][] getParameterInfo() {
       String[][] info = {
        // Parameter Name     Kind of Value  Description
         {"DOBFile",             "URL",         "DemoOBject-file to be used"},
         };
       return info;
   }

   public String getAppletInfo()
   {
      return( "VektoriMoottori v0.1 Copyright(c) Mikko Sairio" );
   }
}