/********************************************************

    vAnim

 ********************************************************/

import java.awt.Image;
import java.awt.image.*;
import java.awt.Component;

public class vAnim
{
   public final static int DIRECT = 0;
   public final static int DIRECT_BUFFERED = 1;
   public final static int BUFFERED = 1;
   public final static int LOOPING = 2;

   int animType = DIRECT;

   int frameCount = 1;
   vWorld myWorld = null;
   vAnimThread animThread = null;

   Component myComponent = null;

   Image[] videoImage = null;
   boolean[] free = null;
   int[] frameId = null;

   public vAnim( int newAnimType, int maxFrames, vWorld animSource, java.awt.Component newComponent )
   {
      animType = newAnimType;
      frameCount = maxFrames;
      myWorld = animSource;
      myComponent = newComponent;

      if (animType == DIRECT) frameCount = 1;

      frameId = new int[ maxFrames ];
      free = new boolean[ maxFrames ];
      videoImage = new Image[ maxFrames ];
      for(int i=0;i<maxFrames;i++)
      {
         frameId[i] = 0;
         free[i] = true;
      }

      animThread = new vAnimThread( 0, frameCount, myWorld, this );
      if (animThread != null) animThread.start();
   }

   int nextAvail = 0;
   int lastReservedSlot = 0;

   public Image getNextAvail()
   {
      Image temp = getImage( nextAvail );
      nextAvail = (++nextAvail) % frameCount;
      return temp;
   }

   public Image getImage( int index )
   {
      index = index % frameCount;

      while (free[index])
      {
//         System.out.println("waiting for a new image at slot " + index);
         try { Thread.sleep(25); } catch(InterruptedException e) {};
      }

      int imageStatus;

      while((imageStatus = myComponent.checkImage( videoImage[index], null ) & ImageObserver.ALLBITS) == 0)
      {
//         System.out.println("imageStatus (of " + index + ") == " + imageStatus);
         if ((imageStatus & (ImageObserver.ABORT | ImageObserver.ERROR)) != 0)
         {
            System.err.println("creation of image failed: checkImage == " + imageStatus );
            return null;
         }
         try { Thread.sleep(25); } catch(InterruptedException e) {};
      };

      lastReservedSlot = index;
      return videoImage[index];
   }

   public void releaseSlot()
   {
      if (animType == LOOPING) return;
//      System.out.println("releasing slot " + lastReservedSlot);
//      if (free[lastReservedSlot]) System.out.println("(already free!)");
      free[lastReservedSlot] = true;
   }

   public void setNewImage( int id, Image newImage )
   {
      int index = id % frameCount;
      while(!free[index])
      {
//         System.out.println("waiting for slot " + index + " to be free");
         try { Thread.sleep(25); } catch(InterruptedException e) {};
      }

      if (videoImage[index] != null) videoImage[index].flush();
      videoImage[index] = newImage;

      myComponent.prepareImage( videoImage[index], null );

//      System.out.println("image at slot " + index + " is ready");
      frameId[index] = id;
      free[index] = false;
   }

   public void setNewImageArray( int id, int[] videoBuffer )
   {
      setNewImage( id, myComponent.createImage ( new MemoryImageSource(
            myWorld.width, myWorld.height, videoBuffer, 0, myWorld.width )));
   }
}

