/********************************************************

   vObject

 ********************************************************/

class vObject extends vMatrix
{

   public vMatrix orientation = null;  // ei muuten saa olla null koskaan!

   private vNormal normalList = null;
   private vVertex vertexList = null;
   private vFace faceList = null;

   private vNormal activeNormalList = null;
   private vVertex activeVertexList = null;
   public  vFace activeFaceList = null;

   public vObject parentObject = null;

   private int faceCount = 0;
   private int vertexCount = 0;

   public vObject nextObject = null;
   public vObject nextActiveObject = null;

   public int shading = vFace.LINE;    // objektin renderointimalli

   public byte textureId=0, bumpId=0, environmentId=0;

   public vTexture texture;
   public vTexture bump;
   public vTexture environment;

   public int hostId;
   public int objectId;

   public int timeStamp = 0;

   public int radius;            // objektin sde (nkyvyyden ja
                                 // osuman tunnistus)

//   private static float[] sin = calcSin();

   public float x, y, z;
   public float xa, ya, za;      // turhia...
   public float rx, ry, rz;      // obun paikka maailmassa

   public vObject(int id, vObject objectList)
   {
      objectId = id;
      nextObject = objectList;
   }

   public void setFaces(int newFaceCount, vFace newFaceList)
   {
      faceList = newFaceList;
      faceCount = newFaceCount;
   }

   public void setVertices(int newVertexCount, vVertex newVertexList)
   {
      vertexList = newVertexList;
      vertexCount = newVertexCount;
   }

/********************************************************

We need three vectors (points):

        - from is the position of the camera.
        - to is the point the camera is looking at.
        - up is the up vector of the camera.

Three vectors will be calculated:

             to - from
        b = -----------
            |to - from|

             (b,up)b - up
        e = --------------
            |(b,up)b - up|

             exb
        f = -----
            |exb|

The resulting matrix is:

        +                  ++                     +
         f    f    f    0  1    0    0    -to  
          x    y    z                        x 
                                               
         e    e    e    0  0    1    0    -to  
          x    y    z                        y 
                                               
         b    b    b    0  0    0    1    -to  
          x    y    z                        z 
                                               
         0    0    0    1  0    0    0    1    
        +                  ++                     +

                              +   +
                               f 
                                 
We don't need translation, so  e  is enough here.
                                 
                               b 
                              +   +


 ********************************************************/

   public vMatrix faceObject(vObject target)
   {
      return faceTarget(target.rx, target.ry, target.rz);
   }

   public vMatrix faceTarget(float dx, float dy, float dz)
   {
      float bx = dx - x;
      float by = dy - y;
      float bz = dz - z;

      float dist = (float) (1.0 / Math.sqrt(bx*bx+by*by+bz*bz));

      r20 = bx * dist;                    // 0
      r21 = by * dist;                    // > 0
      r22 = bz * dist;                    // > 0

      float ux = (float) Math.sin(za);    // 0
      float uy = (float) -Math.cos(za);   // -1

      float bu = ux * r20 + uy * r21;     // bu < 0

      bx = bu * r20 - ux;                  // 0
      by = bu * r21 - uy;                  // > 0
      bz = bu * r22;                       // < 0

      dist = (float) (1.0 / Math.sqrt(bx*bx+by*by+bz*bz));

      r10 = bx * dist;
      r11 = by * dist;
      r12 = bz * dist;

      bx = r11 * r22 - r12 * r21;
      by = r12 * r20 - r10 * r22;
      bz = r10 * r21 - r11 * r20;

      dist = (float) (1.0 / Math.sqrt(bx*bx+by*by+bz*bz));

      r00 = bx * dist;
      r01 = by * dist;
      r02 = bz * dist;

      orientation.r00 = r00;
      orientation.r01 = r01;
      orientation.r02 = r02;

      orientation.r10 = r10;
      orientation.r11 = r11;
      orientation.r12 = r12;

      orientation.r20 = r20;
      orientation.r21 = r21;
      orientation.r22 = r22;

      return this;
   }

/********************************************************

   angleRotate()

   - laskee rotaatiomatriisin kiertokulmien mukaan

X = X * (Cz*Cy + Sz*Sx*Sy) +
    Y * (-Sz*Cy + Cz*Sx*Sy) +
    Z * (Cx*Sy)
Y = X * (Sz*Cx) +
    Y * (Cz*Cx) +
    Z * (-Sx)
Z = X * (-Cz*Sy + Sz*Sx*Cy) +
    Y * (Sz*Sy + Cz*Sx*Cy) +
    Z * (Cx*Cy)

 ********************************************************/

   public vMatrix angleRotate()
   {
/*
      int x = (int) (xa*256+0.5);
      int y = (int) (xa*256+0.5);
      int z = (int) (xa*256+0.5);
*/
      float sinx = (float) Math.sin( xa );
      float cosx = (float) Math.cos( xa );

      float siny = (float) Math.sin( ya );
      float cosy = (float) Math.cos( ya );

      float sinz = (float) Math.sin( za );
      float cosz = (float) Math.cos( za );

      r00 = cosy*cosz + sinz*sinx*siny;
      r01 = -sinz*cosy + cosz*sinx*siny;
      r02 = cosx*siny;
      r10 = sinz*cosx;
      r11 = cosz*cosx;
      r12 = -sinx;
      r20 = -cosz*siny + sinz*sinx*cosy;
      r21 = sinz*siny + cosz*sinx*cosy;
      r22 = cosx*cosy;

      orientation.r00 = r00;
      orientation.r01 = r01;
      orientation.r02 = r02;

      orientation.r10 = r10;
      orientation.r11 = r11;
      orientation.r12 = r12;

      orientation.r20 = r20;
      orientation.r21 = r21;
      orientation.r22 = r22;

      return this;
   }

/********************************************************

   quaternionRotate()

   - laskee rotaatiomatriisin kiertokulmien mukaan

X = X * (Cz*Cy + Sz*Sx*Sy) +
    Y * (-Sz*Cy + Cz*Sx*Sy) +
    Z * (Cx*Sy)
Y = X * (Sz*Cx) +
    Y * (Cz*Cx) +
    Z * (-Sx)
Z = X * (-Cz*Sy + Sz*Sx*Cy) +
    Y * (Sz*Sy + Cz*Sx*Cy) +
    Z * (Cx*Cy)

 ********************************************************/

   public vMatrix quaternionRotate()
   {
/*
      int x = (int) (xa*256+0.5);
      int y = (int) (xa*256+0.5);
      int z = (int) (xa*256+0.5);
*/
      float sinx = (float) Math.sin( xa );
      float cosx = (float) Math.cos( xa );

      float siny = (float) Math.sin( ya );
      float cosy = (float) Math.cos( ya );

      float sinz = (float) Math.sin( za );
      float cosz = (float) Math.cos( za );

      r00 = cosy*cosz + sinz*sinx*siny;
      r01 = -sinz*cosy + cosz*sinx*siny;
      r02 = cosx*siny;
      r10 = sinz*cosx;
      r11 = cosz*cosx;
      r12 = -sinx;
      r20 = -cosz*siny + sinz*sinx*cosy;
      r21 = sinz*siny + cosz*sinx*cosy;
      r22 = cosx*cosy;

      orientation.r00 = r00;
      orientation.r01 = r01;
      orientation.r02 = r02;

      orientation.r10 = r10;
      orientation.r11 = r11;
      orientation.r12 = r12;

      orientation.r20 = r20;
      orientation.r21 = r21;
      orientation.r22 = r22;

      return this;
   }

/********************************************************

   prepareObject()

   - varmistaa, ett parentObject on jo ksitelty
      - jos ei, niin kutsuu parentObject.prepareObject() rekursiivisesti
   - testaa tasojen nkyvyyden
   - laittaa nkyvt tasot activeFaceList:iin
(   - laittaa nkyvien tasojen vertexit activeVertexList:iin)
(   - rotatoi ja projisoi activeVertexList:in)
(   - sorttaa activeFaceList:in)

 ********************************************************/

   public vObject prepareObject( int now, vObject prevActiveObject )
   {
//      System.out.println("preparing object " + this );

      if (timeStamp == now)
      {
//         System.out.println("(again)");
         return this;
      }

      timeStamp = now;

      if (parentObject != null)
      {
//         System.out.println("found parent " + parentObject);
         if (parentObject.timeStamp != timeStamp)
         {
//            System.out.println("parent not ready, preparing it");
            prevActiveObject = parentObject.prepareObject( timeStamp, prevActiveObject );
         }

         orientation.dotProduct( parentObject, this );

         rx = (parentObject.r00 * x) +
              (parentObject.r01 * y) +
              (parentObject.r02 * z) +
              parentObject.rx;

         ry = (parentObject.r10 * x) +
              (parentObject.r11 * y) +
              (parentObject.r12 * z) +
              parentObject.ry;

         rz = (parentObject.r20 * x) +
              (parentObject.r21 * y) +
              (parentObject.r22 * z) +
              parentObject.rz;
      }
      else
      {
// tt ei tapahdu koskaan... kaikilla obuilla paitsi viewPortilla
// on parentObject...
         orientation.copyMatrix( this );

         rx = -((r00 * x) +
                (r01 * y) +
                (r02 * z));
         ry = -((r10 * x) +
                (r11 * y) +
                (r12 * z));
         rz = -((r20 * x) +
                (r21 * y) +
                (r22 * z));
      }

      int visibility = visibility();

      activeFaceList = null;

      if (visibility == -1) return prevActiveObject;

//      System.out.println("object in viewcone");

      vVertex.setObject( this );

      vVertex.lx = vFace.lx = vFace.cx = (r00 * rx) +
                 (r10 * ry) +
                 (r20 * rz);

      vVertex.lx = vFace.ly = vFace.cy = (r01 * rx) +
                 (r11 * ry) +
                 (r21 * rz);

      vVertex.lx = vFace.lz = vFace.cz = (r02 * rx) +
                 (r12 * ry) +
                 (r22 * rz);

      vFace.drawMode = shading;
      vFace.texture = texture;

//      System.out.println("camera related to object (rotated...) at ("+vFace.cx+", "+vFace.cy+", "+vFace.cz+")");

      switch (visibility) {
         case 0:                          // unelmatapaus: obu kokonaan
         {                                // ruudulla
            vFace tempFace = faceList;

            while(tempFace != null)
            {
               activeFaceList = tempFace.addIfVisible( activeFaceList );
               tempFace = tempFace.nextFace;
            }
            break;
         }
         case 1:                          // klippaa yl- tai alareunan
         {                                // kanssa
            vFace tempFace = faceList;

            while(tempFace != null)
            {
               activeFaceList = tempFace.addIfVisible_HorizClip( activeFaceList );
               tempFace = tempFace.nextFace;
            }
            break;
         }
         case 2:                          // klippaa sivun kanssa
         {
            vFace tempFace = faceList;

            while(tempFace != null)
            {
               activeFaceList = tempFace.addIfVisible_VertClip( activeFaceList );
               tempFace = tempFace.nextFace;
            }
            break;
         }
         case 4:                          // klippaa syvyyssuunnassa
         {
            vFace tempFace = faceList;

            while(tempFace != null)
            {
               activeFaceList = tempFace.addIfVisible_DepthClip( activeFaceList );
               tempFace = tempFace.nextFace;
            }
            break;
         }
         default:                         // joku yhdistelm (klippaa!!)
         {
            vFace tempFace = faceList;

            while(tempFace != null)
            {
               activeFaceList = tempFace.addIfVisible_Clip( activeFaceList );
               tempFace = tempFace.nextFace;
            }
         }
      }

      vFace tempFace = activeFaceList;

      switch (shading) {
         case vFace.FLAT:
         case vFace.FLAT_TEXTURE:
         {
            while(tempFace != null)
            {
               tempFace = tempFace.prepareFlat();
            }
            break;
         }

         case vFace.GOURAUD:
         case vFace.GOURAUD_TEXTURE:
         {
            while(tempFace != null)
            {
               tempFace = tempFace.prepareGouraud();
            }
            break;
         }

         case vFace.PHONG:
         case vFace.PHONG_TEXTURE:
         {
            while(tempFace != null)
            {
               tempFace = tempFace.preparePhong();
            }
            break;
         }

         default:
      }

      return this;
   }

/********************************************************

   boolean visibility()

   - tarkistaa onko objekti viewconessa

 ********************************************************/

   public int visibility()
   {
//      System.out.println( "object at ("+rx+", "+ry+", "+rz+") related to viewport");

      return 7;
   }
}

